#include <iostream>
#include <unordered_map>
#include <map>
#include <chrono>
#include <random>
#include <string>

// Function to generate random strings
std::string random_string(std::size_t length) {
    const std::string characters = "abcdefghijklmnopqrstuvwxyz";
    std::random_device rd;
    std::mt19937 generator(rd());
    std::uniform_int_distribution<> distribution(0, characters.size() - 1);

    std::string result;
    for (std::size_t i = 0; i < length; ++i) {
        result += characters[distribution(generator)];
    }
    return result;
}

// Function to measure execution time
template<typename Func>
long long measureTime(Func func) {
    auto start = std::chrono::high_resolution_clock::now();
    func();
    auto end = std::chrono::high_resolution_clock::now();
    return std::chrono::duration_cast<std::chrono::microseconds>(end - start).count();
}

int main() {
    const int NUM_ELEMENTS = 1000000;
    const int NUM_SEARCHES = 1000000;

    std::unordered_map<std::string, int> umap;
    std::map<std::string, int> map;

    std::vector<std::string> keys;
    for (int i = 0; i < NUM_ELEMENTS; ++i) {
        keys.push_back(random_string(10));
    }

    // Insertion
    auto insertUnordered = [&]() {
        for (const auto& key : keys) {
            umap[key] = 1;
        }
    };
    auto insertOrdered = [&]() {
        for (const auto& key : keys) {
            map[key] = 1;
        }
    };

    std::cout << "Insertion time (microseconds):" << std::endl;
    std::cout << "unordered_map: " << measureTime(insertUnordered) << std::endl;
    std::cout << "map: " << measureTime(insertOrdered) << std::endl;

    // Search
    auto searchUnordered = [&]() {
        for (int i = 0; i < NUM_SEARCHES; ++i) {
            umap.find(keys[i % NUM_ELEMENTS]);
        }
    };
    auto searchOrdered = [&]() {
        for (int i = 0; i < NUM_SEARCHES; ++i) {
            map.find(keys[i % NUM_ELEMENTS]);
        }
    };

    std::cout << "Search time (microseconds):" << std::endl;
    std::cout << "unordered_map: " << measureTime(searchUnordered) << std::endl;
    std::cout << "map: " << measureTime(searchOrdered) << std::endl;

    return 0;
}
